package se.nekotronic.readypinger;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.text.format.Time;

public class PingThread extends Thread {
	private ReadyPingActivity parent;
	private boolean please_stop_was_called = false;
	private int numberOfPings;
	
	public PingThread(ReadyPingActivity parent, int numberOfPings) {
		this.parent = parent;
		this.numberOfPings = numberOfPings;
	}
	
	public void please_stop() {
		please_stop_was_called = true;
	}
	
	public void run() {
		try {
			parent.print("Connecting to " + ReadyPingActivity.host + "...");
			Socket socket = new Socket(ReadyPingActivity.host, ReadyPingActivity.port);
			parent.print("Connected.");
			if (please_stop_was_called)
				return;
			 BufferedReader from_server = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			 PrintWriter to_server = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);
			 // true: PrintWriter is line buffered
			 to_server.println("HELLO");
			 String reply = from_server.readLine();
			 if (! reply.equals("HELLO")) {
				 parent.warning("Wrong reply from server: '" + reply + "'");
				 please_stop();
			 }

			 int max = 0, min = 0, sum = 0, count = 0;
			 int[] alltimes = new int[numberOfPings];
			 for (int i = 0; i < numberOfPings ; ++i) {
				 long before = System.currentTimeMillis();
				 to_server.println("PING " + i);
				 reply = from_server.readLine();
				 long after = System.currentTimeMillis();
			     
				 if (! reply.startsWith("PONG ")) {
					 parent.warning("Wrong reply from server: '" + reply + "'");
				 }
				 else {
				     long millis = after - before;
				     alltimes[i] = (int)millis;
					 parent.print(millis + " ms");
				     if (i == 1 || millis < min)
				    	 min = (int)millis;
				     if (i == 1 || millis > max)
				    	 max = (int)millis;
				     sum += millis;
				     ++count;
				 }
			 }
			 parent.print("Pinging done.");
			 
			 Time now = new Time();
		     now.setToNow();
		     String timeString = now.format("%Y-%m-%d %H:%M:%S");
		     String report = "time = " + timeString +
		    		 "; lat = " + parent.getLatestLatitude() +
		    		 "; long = " + parent.getLatestLongitude() +
		    		 "; count = " + count +
		    		 "; avg = " + (1.0 * sum / count) + "; min = " + min +
		    		 "; max = " + max +
		    		 "; alltimes = { ";
		     for (int i = 0; i < count; ++i)
		    	 report += alltimes[i] + " ";
		     report += " }";
		     String deviceinfo = " -- OS Version: " + System.getProperty("os.version") + "(" + android.os.Build.VERSION.INCREMENTAL + "); " +
		           "OS API Level: " + android.os.Build.VERSION.SDK + "; " +
		           "Device: " + android.os.Build.DEVICE + "; " +
		           "Model (and Product): " + android.os.Build.MODEL + " ("+ android.os.Build.PRODUCT + ")";
		     
		 	/*
		     You can use getSubtype() for more details. Check out slide 9 here: http://dl.google.com/io/2009/pres/W_0300_CodingforLife-BatteryLifeThatIs.pdf
			*/
		     ConnectivityManager cm = (ConnectivityManager) parent.getSystemService(Context.CONNECTIVITY_SERVICE);
		     // TelephonyManager tm = null;
		     // Skip if no connection, or background data disabled
		     NetworkInfo info = cm.getActiveNetworkInfo();
		     if (info != null) {
		    	 deviceinfo += "; Network Type: " + info.getTypeName() +
		    			 	   "; Network Subtype: " + info.getSubtypeName();
		     }

		     /*
		     // Only update if WiFi or 3G is connected and not roaming
		     int netType = info.getType();
		     int netSubtype = info.getSubtype();
		     if (netType == ConnectivityManager.TYPE_WIFI) {
		         return info.isConnected();
		     } else if (netType == ConnectivityManager.TYPE_MOBILE
		         && netSubtype == TelephonyManager.NETWORK_TYPE_UMTS
		         && !tm.isNetworkRoaming()) {
		             return info.isConnected();
		     } else {
		         return false;
		     }
		     */
		     
		     report += deviceinfo;
		     
			 to_server.println("REPORT " + report);
			 parent.print("REPORT sent: " + "REPORT " + report);
			 
			 to_server.close();
			 from_server.close();
			 parent.done();
		}
		catch (UnknownHostException e) {
			parent.warning("Unknown host: " + e.getMessage());
		} catch (IOException e) {
			parent.warning("I/O Exception: " + e.getMessage());
		}
	}
}
